﻿namespace aspx2razor
{
    using System;
    using System.ComponentModel.Composition;
    using System.ComponentModel.Composition.Hosting;
    using System.IO;
    using System.Text;
    using Telerik.RazorConverter;
    using Telerik.RazorConverter.Razor.DOM;

    class Program
    {
        [Import]
        private IWebFormsParser Parser
        {
            get;
            set;
        }

        [Import]
        private IWebFormsConverter<IRazorNode> Converter
        {
            get;
            set;
        }

        [Import]
        private IRenderer<IRazorNode> Renderer
        {
            get;
            set;
        }

        private static void Main(string[] args)
        {
            var p = new Program();
            p.Run(args);            
        }

        private Program()
        {
            var catalog = new AssemblyCatalog(typeof(IWebFormsParser).Assembly);
            var container = new CompositionContainer(catalog);
            container.ComposeParts(this);

        }

        
        public void Run(string[] args)
        {
            if (args.Length < 1)
            {
                DisplayUsage();
                return;
            }

            string rzfile = @"C:\Users\Raj\Desktop\Labs\RazorConverter\telerik-razor-converter-e303d27\SampleAspxToRazorApp\SampleAspxToRazorApp\Views\Account\LogOn.aspx";
            string directory = @"C:\Users\Raj\Desktop\Labs\RazorConverter\telerik-razor-converter-e303d27\SampleAspxToRazorApp\SampleAspxToRazorApp\Views\Account\";

            var inputDirectory = Path.GetDirectoryName(rzfile);
            if (string.IsNullOrEmpty(inputDirectory))
            {
                inputDirectory = Directory.GetCurrentDirectory();
            }

            var inputFilter = Path.GetFileName(rzfile);

            var outputDirectory = (args.Length == 2) ? directory : inputDirectory;
            foreach (var file in Directory.GetFiles(inputDirectory, inputFilter))
            {
                Console.Write("Converting {0}... ", Path.GetFileName(rzfile));
                
                var webFormsPageSource = File.ReadAllText(file, Encoding.UTF8);
                var webFormsDocument = Parser.Parse(webFormsPageSource);
                var razorDom = Converter.Convert(webFormsDocument);
                var razorPage = Renderer.Render(razorDom);

                var outputFile = Path.Combine(outputDirectory, Path.GetFileNameWithoutExtension(file) + ".cshtml");
                File.WriteAllText(outputFile, razorPage, Encoding.UTF8);
                
                Console.WriteLine("done");
            }

        }

        private void DisplayUsage()
        {
            Console.WriteLine("Converts WebForms pages (.aspx, .ascx) into a Razor views (.cshtml)");
            Console.WriteLine("Usage: aspx2razor <input file / wildcard> [output-directory]");
        }
    }
}
